/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pplite-config.h"

#include "Index_Set.hh"
#include "ascii_dump_load.hh"

namespace pplite {

void
Index_Set::ascii_dump(std::ostream& os) const {
  auto sz = size();
  os << sz << " : ";
  if (sz == 0) {
    os << "{}\n";
    return;
  }
  bool first = true;
  for (auto i : *this) {
    os << (first ? "{ " : ", ") << i;
    first = false;
  }
  os << " }\n";
}

bool
Index_Set::ascii_load(std::istream& is) {
  clear();
  dim_type num = 0;
  if (not ((is >> num) && (num >= 0)))
    return false;
  if (not ascii_load_string(is, ":"))
    return false;
  if (num == 0)
    return ascii_load_string(is, "{}");

  if (not ascii_load_string(is, "{"))
    return false;
  dim_type idx = not_a_dim();
  if (not ((is >> idx) && (idx >= 0)))
    return false;
  set(idx);
  dim_type old_idx = idx;
  for (dim_type i = 1; i < num; ++i) {
    if (not ascii_load_string(is, ","))
      return false;
    if (not ((is >> idx) && (idx >= 0) && (idx > old_idx)))
      return false;
    set(idx);
    old_idx = idx;
  }
  return ascii_load_string(is, "}");
}

} // namespace pplite

