/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pplite-config.h"
#include "Poly_Stats.hh"

#include <iostream>
#include <iomanip>
#include <cstring>

pplite::AbsOp_Stats pplite::absop_stats;

namespace pplite {

void
AbsOp_Stats::dump_op(std::ostream& os, AbsOp op, bool dump_times) const {
  auto op_val = static_cast<unsigned>(op);
  using namespace std;
  os << "\n" << absop_names[op_val] << " ";
  const Time_Stats& ts = time_stats[op_val];
  os << ts.counter;
  if (dump_times) {
    os << " ";
    Clock::print(os, ts.secs, ts.usecs);
  }
  os << flush;
}

void
AbsOp_Stats::dump(std::ostream& os, bool dump_times) const {
  os << "=== Dumping time stats for pplite::Poly's operations ===\n";
  for (unsigned i = 0; i < absop_size; ++i)
    dump_op(os, static_cast<AbsOp>(i), dump_times);
  os << std::endl;
  dump_overall_time(os);
  os << std::endl;
}

void
AbsOp_Stats::dump_overall_time(std::ostream& os) const {
  time_t secs = 0;
  time_t usecs = 0;
  for (const auto& ts : time_stats) {
    secs += ts.secs;
    usecs += ts.usecs;
    if (usecs >= 1000000) {
      ++secs;
      usecs %= 1000000;
    }
  }
  os << "Total AbsOp time: ";
  Clock::print(os, secs, usecs);
  os << std::endl;
}

void
AbsOp_Stats::reset() {
  for (auto& ts : time_stats) {
    ts.counter = 0;
    ts.secs = 0;
    ts.usecs = 0;
  }
}

AbsOp_Stats::~AbsOp_Stats() {
  if (noisy_dtor)
    dump(std::cerr, true);
}

} // namespace pplite
